public class Person implements Comparable<Person> {
    
    private String firstname, surname;
    
    public Person(String firstname, String surname) {
        if (firstname == null) throw new NullPointerException("firstname must be given");
        if (surname == null) throw new NullPointerException("surname must be given");
        this.firstname = firstname;
        this.surname = surname;
    }
    
    public String getSurname() {
        return surname;
    }
    
    public String getFirstname() {
        return firstname;
    }
    
    @Override
    public String toString() {
        return firstname + " " + surname;
    }
    
    // Both equals AND hashCode MUST BE OVERRIDEN when using HashSet/HashMap
    @Override 
    public boolean equals(Object o) {
        if(o == null) return false;
        if(this.getClass() != o.getClass()) return false;
        Person p = (Person)o;
        return this.firstname.equals(p.firstname) && this.surname.equals(p.surname);
    }
    
    // If you just override equals BUT NOT hashCode
    // then HashSet will not work!!!
    // Try to comment hashCode() out...
    @Override
    public int hashCode() {
        return firstname.hashCode() ^ surname.hashCode();
    }
    
    @Override
    public int compareTo(Person p) {
        if(p == null) throw new NullPointerException("Cannot compare to null");
        
        int c = this.surname.compareTo(p.surname);
        if(c != 0) return c;
        
        return this.firstname.compareTo(p.firstname);
    }
}